import sys

import logging
from logging import config as log_conf
from collections import OrderedDict
from jsonformatter import JsonFormatter

from pygments import highlight
from pygments.lexers.data import JsonLexer
from pygments.formatters.terminal import TerminalFormatter

from . import fn


RECORD_CUSTOM_FORMAT = OrderedDict([
    ('level', 'levelname'),
    ('msg', 'message'),
    ('name', 'name'),
    ('file', 'pathname'),
    ('module', 'module'),
    ('line', 'lineno'),
    ('func', 'funcName'),
    ('datetime', 'asctime'),
])


class LogFormat(JsonFormatter):
    """Log formatter."""

    def format(self, record: logging.LogRecord):
        """Overload."""
        # from extra instead
        record.exc_info = None
        record.exc_text = None

        try:
            text_record = super().format(record)
        except Exception:
            return str(record)

        return highlight(text_record, JsonLexer(), TerminalFormatter())


def assign_logger_handlers() -> None:
    """Logger handler re-assign."""
    loggers = (
        'sqlalchemy.engine',
        'sqlalchemy.pool',
        'uvicorn.access',
        'uvicorn.error',
    )

    for ext_name in loggers:
        _ext_log = logging.getLogger(ext_name)
        _ext_log.setLevel(log_level)
        _ext_log.addHandler(handler)


log_level = logging.DEBUG if fn.is_debug() else logging.ERROR

handler = logging.StreamHandler()
handler.setFormatter(
    LogFormat(
        RECORD_CUSTOM_FORMAT,
        # record_custom_attrs= TODO
        ensure_ascii=False,
        mix_extra=True,
        indent=4,
        mix_extra_position='tail',
    ),
)

handler.setLevel(log_level)
log_conf.dictConfig({
    'version': 1,
    'disable_existing_loggers': False,
    'loggers': {
        'uvicorn': {'propagate': False},
        'uvicorn.access': {'propagate': False},
        'uvicorn.error': {'propagate': True},
    },
})


log = logging.getLogger('service')
log.setLevel(log_level)
log.addHandler(handler)

assign_logger_handlers()


def exception_hook(exc_type, exc_value, exc_traceback):
    """Handle exception."""
    exc_info = (exc_type, exc_value, exc_traceback)
    if issubclass(exc_type, KeyboardInterrupt):
        sys.__excepthook__(*exc_info)
        return

    extra = {
        'type': exc_type,
        'value': exc_value,
        'traceback': exc_traceback,
    }

    try:
        log.exception('Uncaught exception', extra=extra, exc_info=exc_info)
    except Exception:
        print(exc_info)


sys.excepthook = exception_hook


__all__ = 'log', 'handler'
