import sys

from fastapi import Request
from fastapi.responses import JSONResponse
from starlette.types import ASGIApp
from starlette.middleware.base import BaseHTTPMiddleware

import sentry_sdk

from tblib import Traceback

from core import fn, log


class ServiceMiddleware(BaseHTTPMiddleware):
    """Service - maintenance middleware."""

    def __init__(self, app: ASGIApp):
        """Init."""
        super().__init__(app)
        self.app = app
        self.debug = fn.is_debug()

    async def dispatch(self, request: Request, call_next):
        """Overload dispatch."""
        try:
            response = await call_next(request)
        except Exception as fault:
            _ex, _ev, _et = sys.exc_info()
            with sentry_sdk.push_scope() as scope:
                scope.set_context('request', request)
                log.debug(
                    fault,
                    extra={
                        'type': str(fault.__class__.__name__),
                        'value': str(fault),
                        'traceback': Traceback(_et).as_dict(),
                    },
                    exc_info=True,
                )
                sentry_sdk.capture_exception(fault)
                return JSONResponse(status_code=500)
        else:
            if self.debug:
                log.debug('Trace request')

        return response
